#!/bin/bash

# Environment variables
py="/usr/bin/python2"
waf_dir="src/ns-allinone-3.16/ns-3.16"
trace_dir="traces"
cwd_dir="../../../${trace_dir}" # relative to $waf_dir
ns3_script="scratch/diffserv-simulator"

VoipFlag=0
VtFlag=0
DS=0
Policy=0
MdrrMode="ALTERNATE"
EFsize=100
AF1size=100
AF2size=100
AF3size=100
AF4size=100
BEsize=100
EFweight=1
AF1weight=1
AF2weight=1
AF3weight=1
AF4weight=1
BEweight=1

# Functions
function usage()
{
  echo ""
  echo "usage: $0 test duration"
  echo ""
  echo "test    : Name of test to run (test{1-${total_tests}})"
  echo "duration: Simulation duration in seconds (>= 1)"
  exit
}

function test_run()
{
  echo "########################"
  echo "Test: ${TestName}"
  echo "------------------------"
  if [[ ${VoipFlag} -eq 0 ]]
  then
    echo "VoIP        : disabled"
  else
    echo "VoIP        : enabled"
  fi
  if [[ ${VtFlag} -eq 0 ]]
  then
    echo "Video stream: disabled"
  else
    echo "Video stream: enabled"
  fi
  if [[ ${DS} -eq 0 ]]
  then
    echo "DiffServ    : disabled"
  else
    echo "DiffServ    : enabled"
    echo "MDRR        : ${MdrrMode}"
    if [[ ${Policy} -gt 0 ]]
    then
      echo "Policy      : ${Policy}"
    else
      echo "Policy      : disabled"
    fi
    echo "------------------------"
    echo "EF  size: ${EFsize}"
    echo "AF1 size: ${AF1size}"
    echo "AF2 size: ${AF2size}"
    echo "AF3 size: ${AF3size}"
    echo "AF4 size: ${AF4size}"
    echo "BE  size: ${BEsize}"
    echo "------------------------"
    echo "EF  weight: ${EFweight}"
    echo "AF1 weight: ${AF1weight}"
    echo "AF2 weight: ${AF2weight}"
    echo "AF3 weight: ${AF3weight}"
    echo "AF4 weight: ${AF4weight}"
    echo "BE  weight: ${BEweight}"
  fi
  echo "########################"

  cd ${waf_dir}
  ${py} waf --run "${ns3_script} ${params}" --cwd=${cwd_dir}
  cd ../../..
}

function test1()
{
  TestName="Test1"
  VoipFlag=1
  VtFlag=0
  DS=0
  params="--SimName=${TestName} --SimDuration=${Duration} \
          --VoipFlag=${VoipFlag} --VtFlag=${VtFlag} \
          --DS=${DS} --Policy=${Policy} --MdrrMode=${MdrrMode}"
}

function test2()
{
  TestName="Test2"
  VoipFlag=1
  VtFlag=1
  DS=0
  params="--SimName=${TestName} --SimDuration=${Duration} \
          --VoipFlag=${VoipFlag} --VtFlag=${VtFlag} \
          --DS=${DS} --Policy=${Policy} --MdrrMode=${MdrrMode}"
}

function test3()
{
  TestName="Test3"
  VoipFlag=1
  VtFlag=1
  DS=1
  Policy=0
  params="--SimName=${TestName} --SimDuration=${Duration} \
          --VoipFlag=${VoipFlag} --VtFlag=${VtFlag} \
          --DS=${DS} --Policy=${Policy} --MdrrMode=${MdrrMode}"
}

function test4()
{
  TestName="Test4"
  VoipFlag=1
  VtFlag=1
  DS=1
  Policy=1
  EFweight=1
  AF1weight=1
  AF2weight=1
  AF3weight=1
  AF4weight=1
  BEweight=1
  MdrrMode="STRICT"
  params="--SimName=${TestName} --SimDuration=${Duration} \
          --VoipFlag=${VoipFlag} --VtFlag=${VtFlag} \
          --DS=${DS} --Policy=${Policy} --MdrrMode=${MdrrMode}"
}

function test5()
{
  TestName="Test5"
  VoipFlag=1
  VtFlag=1
  DS=1
  Policy=2
  EFweight=1
  AF1weight=1
  AF2weight=1
  AF3weight=1
  AF4weight=1
  BEweight=1
  MdrrMode="STRICT"
  params="--SimName=${TestName} --SimDuration=${Duration} \
          --VoipFlag=${VoipFlag} --VtFlag=${VtFlag} \
          --DS=${DS} --Policy=${Policy} --MdrrMode=${MdrrMode}"
}

function test6()
{
  TestName="Test6"
  VoipFlag=1
  VtFlag=1
  DS=1
  Policy=3
  EFweight=10
  AF1weight=5
  AF2weight=1
  AF3weight=1
  AF4weight=1
  BEweight=1
  MdrrMode="ALTERNATE"
  MTU=552
  params="--SimName=${TestName} --SimDuration=${Duration} --DS=${DS} \
          --EFweight=${EFweight} --AF1weight=${AF1weight} \
          --AF2weight=${AF2weight} --AF3weight=${AF3weight} \
          --AF4weight=${AF4weight} --BEweight=${BEweight} \
          --VoipFlag=${VoipFlag} --VtFlag=${VtFlag} \
          --Policy=${Policy} --MdrrMode=${MdrrMode} --MTU=${MTU}"
}

total_tests=6

function tests()
{
  case ${TestInput} in
  test1)
    test1
    test_run
  ;;
  test2)
    test2
    test_run
  ;;
  test3)
    test3
    test_run
  ;;
  test4)
    test4
    test_run
  ;;
  test5)
    test5
    test_run
  ;;
  test6)
    test6
    test_run
  ;;
  all)
    test1
    test_run

    test2
    test_run

    test3
    test_run

    test4
    test_run

    test5
    test_run

    test6
    test_run
  ;;
  *)
    echo "Unsupported test: ${TestInput}"
    usage
  ;;
  esac
}

# main
if [[ $# -eq 1 ]]
then
  echo "No duration specfied."
  usage
elif [[ $# -eq 2 ]]
then
  echo "Checking output directory: ${trace_dir}"
  if [[ ! -d ${trace_dir} ]]
  then
    echo "Not found. Creating ..."
    mkdir -p traces
    echo "Ok."
  else
    echo "Ok."
  fi

  TestInput=$1
  Duration=$2
  tests
else
  usage
fi
