/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2013 University of Patras,
 * Department of Computer Engineering & Informatics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 * Author: Christos Nianiakas <cnianiak_at_gmail_dot_com>
 */

#ifndef __SLA_HELPER_H__
#define __SLA_HELPER_H__

#include "diffserv-helper.h"
#include "ns3/diffserv-queue.h"
#include "ns3/policy.h"
#include "ns3/object-factory.h"

namespace ns3 {

/**
 * \brief A DiffServ architecture implementation
 *
 * SlaHelper class helps creating SLAs for flows
 */

class SlaHelper : public Object
{
public:
  static TypeId GetTypeId (void);

/**
 * \brief SlaHelper Constructor
 *
 * Creates an SlaHelper instance
 */
  SlaHelper ();
  virtual ~SlaHelper ();

/**
 * Setup a flow
 *
 * \param srcAddress Source address of flow
 * \param destAddress Destination address of flow
 * \param protocol Protocol used by the flow
 * \param srcPort Source port used by the flow
 * \param destPort Destintaion port used by the flow
 */
  void SetFlow (Ipv4Address srcAddress,
                Ipv4Address destAddress,
                uint8_t protocol,
                uint16_t srcPort,
                uint16_t destPort);
/**
 * Setup the type of policy
 *
 * \param type Type of policy to use
 */
  void SetType (Policy::Type);
/**
 * Setup the metrics used by the policer
 *
 * \param metrics Metrics used by the policer
 */
  void SetMetrics (Policy::Metrics metrics);
/**
 * Set the policy for each conformance level
 */
  void SetGreen (Policy::Mode mode);
  void SetGreen (Policy::Mode mode, Ipv4Header::DscpType dscp);
  void SetYellow (Policy::Mode mode);
  void SetYellow (Policy::Mode mode, Ipv4Header::DscpType dscp);
  void SetRed (Policy::Mode mode);
  void SetRed (Policy::Mode , Ipv4Header::DscpType dscp);
/**
 * Install created SLA into a Node
 *
 * \param node Node to install the SLA
 */
  void Install (Ptr<Node> node);

private:
  bool flowOK, metricsOK;
  bool green, yellow, red;
  DiffServ::Flow m_Flow;
  Policy::Actions m_Actions;
  Policy::Metrics m_Metrics;
};

} // namespace ns3

#endif /* __SLA_HELPER_H__ */
