/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2013 University of Patras,
 * Department of Computer Engineering & Informatics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 * Author: Christos Nianiakas <cnianiak_at_gmail_dot_com>
 */

#ifndef __TRAFFIC_HELPER_H__
#define __TRAFFIC_HELPER_H__

#include "ns3/object.h"
#include "ns3/node.h"
#include "ns3/ipv4-header.h"
#include "ns3/data-rate.h"

namespace ns3 {

/* ... */

class VoIpHelper : public Object
{
public:
  static TypeId GetTypeId (void);

  VoIpHelper ();
  virtual ~VoIpHelper ();

  void SetDuration (double, double);
  void SetDestination (Ipv4Address, uint16_t);
  void Install (Ptr<Node>);

private:
  bool m_Duration;
  double m_Start;
  double m_Stop;
  bool m_Destination;
  Ipv4Address m_Address;
  uint16_t m_Port;
  DataRate m_Rate;
  uint32_t m_PacketSize;
};

class VideoTraceHelper : public Object
{
public:
  static TypeId GetTypeId (void);

  VideoTraceHelper ();
  virtual ~VideoTraceHelper ();

  void SetDuration (double, double);
  void SetDestination (Ipv4Address, uint16_t);
  void SetTrace (std::string);
  void Install (Ptr<Node>);

private:
  bool m_Duration;
  double m_Start;
  double m_Stop;
  bool m_Destination;
  Ipv4Address m_Address;
  uint16_t m_Port;
  bool m_Trace;
  std::string m_TraceSource;
  uint32_t m_MaxPacketSize;
};

} // namespace ns3

#endif /* __TRAFFIC_HELPER_H__ */
